/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2014 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Common definitions
 *
 * ************************************************************************
 */

"use strict";

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

// show debug message in browser log window
const DEBUG                       = false;
const CK_PREF_VERSION             = 2; // Change only if the preferences need to be updated

// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date
const DATE_MAX                    = 100000000 * 86400000; // in miliseconds

// Preferences constants:
const PREF_BRANCH                 = "extensions.cookiekeeper.";
const PREF_VERSION                = "extensions.cookiekeeper.version";
const PREF_CLEAR_COOKIES_AT_EXIT  = "extensions.cookiekeeper.clear_at_exit.cookies";
const PREF_CLEAR_DOM_AT_EXIT      = "extensions.cookiekeeper.clear_at_exit.dom";
const PREF_NO_ASK_AGAIN           = "extensions.cookiekeeper.no_ask_again";
// Protected cookie string format:
// host;name;path|host;name;path|...
const PREF_PROT_COOKIES           = "extensions.cookiekeeper.protected";
const PROT_SEP_COOKIE             = "|";
const PROT_SEP_DATA               = ";";
// show the boxFixPrefClear if mozilla pref can messed up our config:
const PREF_WARN_MOZ               = "extensions.cookiekeeper.warn_moz_pref";
// Moz App names:
const FIREFOX                     = "Firefox";
const THUNDERBIRD                 = "Thunderbird";
const SEAMONKEY                   = "SeaMonkey";

const CK_BUTTON_ID                = "cookiekeeper-button";
const CK_MENU_ID                  = "cookiekeeper-menu";
const CK_WINDOWS_ID               = ["cookieKeeperWindow",
                                    "cookieKeeperPrefsWindow",
                                    "cookieKeeperEditorCookie"];
const CK_WINDOW_MAIN_IDX          = 0;
const CK_WINDOW_PREFS_IDX         = 1;

const CK_PREFPANE_OPTIONS         = "cookiekeeper-prefs-pane-cookiekeeper";
const CK_PREFPANE_PCOOKIES        = "cookiekeeper-prefs-pane-pcookies";
const CK_PREFPANE_PERMISSIONS     = "cookiekeeper-prefs-pane-permissions";
const CK_PREFPANE_BROWSER         = "cookiekeeper-prefs-pane-browser";

const CK_MODULE_NAME              = "[CookieKeeper]";

// Observer ProtectedCookies:
const CK_OBS_PCOOKIE_TOPIC        = "pcookie-changed";
const CK_OBS_PCOOKIE_STARTUP      = "startup";  // The add-on is starting
const CK_OBS_PCOOKIE_SHUTDOWN     = "shutdown"; // The add-on is shutdown
const CK_OBS_PCOOKIE_ADD          = "added";
const CK_OBS_PCOOKIE_DEL          = "deleted";

// Mozilla preferences:
// from mozilla-release/netwerk/cookie/nsCookieService.cpp (Firefox 24.0):
// values for network.cookie.cookieBehavior:
const MOZ_BEHAVIOR_ENABLED  = 0; // allow all cookies (default)
const MOZ_BEHAVIOR_SRV_ONLY = 1; // reject all third-party cookies
const MOZ_BEHAVIOR_DISABLED = 2; // reject all cookies
const MOZ_BEHAVIOR_P3P      = 3; // reject third-party cookies unless the eTLD already has at least one cookie
// from mozilla-release/extensions/cookie/nsCookiePermission.cpp (Firefox 24.0)
// values for network.cookie.lifetimePolicy:
const MOZ_ACCEPT_NORMALLY   = 0; // accept normally
const MOZ_ASK_BEFORE_ACCEPT = 1; // ask before accepting
const MOZ_ACCEPT_SESSION    = 2; // downgrade to session
const MOZ_ACCEPT_FOR_N_DAYS = 3; // limit lifetime to N days

/*
// pref string constants
static const char kPrefCookieBehavior[]     = "network.cookie.cookieBehavior";
static const char kPrefMaxNumberOfCookies[] = "network.cookie.maxNumber";
static const char kPrefMaxCookiesPerHost[]  = "network.cookie.maxPerHost";
static const char kPrefCookiePurgeAge[]     = "network.cookie.purgeAge";
static const char kPrefThirdPartySession[]  = "network.cookie.thirdparty.sessionOnly";

// default limits for the cookie list. these can be tuned by the
// network.cookie.maxNumber and network.cookie.maxPerHost prefs respectively.
static const uint32_t kMaxNumberOfCookies = 3000;
static const uint32_t kMaxCookiesPerHost  = 150;
static const uint32_t kMaxBytesPerCookie  = 4096;
static const uint32_t kMaxBytesPerPath    = 1024;
*/

const APP_MOZ_CONST = {
  // Firefox
  "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}": {
    NAME:                       FIREFOX,
    XUL:                        "chrome://cookiekeeper/content/prefs.firefox.xul",
    PRIVATEBROWSING:            "browser.privatebrowsing.autostart",
    SANITIZE_ON_SHUTDOWN:       "privacy.sanitize.sanitizeOnShutdown",
    COOKIES_CLEAR_ON_SHUTDOWN:  "privacy.clearOnShutdown.cookies",
    COOKIES_BEHAVIOR:           "network.cookie.cookieBehavior",
    COOKIES_LIFETIME:           "network.cookie.lifetimePolicy",
    COOKIES_LIFETIME_DAYS:      "network.cookie.lifetime.days",
    WINDOW_TYPE:                "navigator:browser", // main window attribute "windowtype"
    TOOLBAR:                    "nav-bar",  // default toolbar
    TOOLBOX:                    "navigator-toolbox",
    PALETTE:                    "BrowserToolbarPalette",
    TOOLSMENU:                  "menu_ToolsPopup"
  },
  // SeaMonkey
  "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}": {
    NAME:                       SEAMONKEY,
    XUL:                        "chrome://cookiekeeper/content/prefs.seamonkey.xul",
    SANITIZE_ON_SHUTDOWN:       "privacy.sanitize.sanitizeOnShutdown",
    COOKIES_CLEAR_ON_SHUTDOWN:  "privacy.item.cookies",
    COOKIES_BEHAVIOR:           "network.cookie.cookieBehavior",
    COOKIES_LIFETIME:           "network.cookie.lifetimePolicy",
    COOKIES_LIFETIME_DAYS:      "network.cookie.lifetime.days",
    WINDOW_TYPE:                "navigator:browser",
    TOOLBAR:                    "nav-bar",
    TOOLBOX:                    "navigator-toolbox",
    PALETTE:                    "BrowserToolbarPalette",
    TOOLSMENU:                  "taskPopup"
  },
  // Thunderbird
  "{3550f703-e582-4d05-9a08-453d09bdfdc6}": {
    NAME:                       THUNDERBIRD,
    XUL:                        "chrome://cookiekeeper/content/prefs.thunderbird.xul",
    COOKIES_BEHAVIOR:           "network.cookie.cookieBehavior",
    COOKIES_LIFETIME:           "network.cookie.lifetimePolicy",
    COOKIES_LIFETIME_DAYS:      "network.cookie.lifetime.days",
    WINDOW_TYPE:                "mail:3pane",
    TOOLBAR:                    "mail-bar3",
    TOOLBOX:                    "mail-toolbox",
    PALETTE:                    "MailToolbarPalette",
    TOOLSMENU:                  "taskPopup"
  }
}

/**
 * Logging facility.
 */
var log = {

  d: function(args) {
    if (DEBUG) {
      try {
        console.log(CK_MODULE_NAME, args);
      } catch (ex) {
        // The famous "try catch, do nothing"!
        // Ignore message
        Cu.reportError(args);
      }
    }
  },

  i: function(args) {
    if (DEBUG) {
      try {
        console.info(CK_MODULE_NAME, args);
      } catch (ex) {
        // Ignore message
        Cu.reportError(args);
      }
    }
  },

  w: function(args) {
    if (DEBUG) {
      try {
        console.warn(CK_MODULE_NAME, args);
      } catch (ex) {
        // Ignore message
        Cu.reportError(args);
      }
    }
  },

  e: function(args) {
    if (DEBUG) {
      try {
        console.error.apply(CK_MODULE_NAME, args);
      } catch (ex) {
        // Ignore message
        Cu.reportError(args);
      }
    }
  }
};
