/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2014 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Preferences: Show/remove protected cookies list.
 *
 * ************************************************************************
 */

Cu.import("resource://cookiekeeper/cookiekeeper.jsm");

var gCookieKeeperProtCookiesPrefs = {
  _buttonRemove : null,
  _treeView     : null,
  _treeChildren : null,
  _treeCols     : [],
  _pCookies     : [],
  _sortColumn   : 0,
  _sortOrderAsc : false,

  init: function() {
    this._buttonRemove = document.getElementById("buttonRemove");
    this._treeView     = document.getElementById("treeProtectedCookies");
    this._treeChildren = document.getElementById("treeProtCookiesChildren");
    this._treeCols.push(document.getElementById("tcDomain"));
    this._treeCols.push(document.getElementById("tcName"));
    this._treeCols.push(document.getElementById("tcPath"));

    this._pCookies = CookieKeeper.loadProtectedCookies();

    if (this._pCookies.length == 0) {
      this._buttonRemove.disabled = true;
      this._treeView.disabled     = true;
    } else {
      this._treeView.seltype = "single";
      this.onSort(null, 0);
    }

    if (this._pCookies.length > 0) {
      this._treeView.view.selection.select(0);
    }
  },

  _fillList: function() {
    for (var i=0; i<this._pCookies.length; i++) {
      let pc   = this._pCookies[i];
      let item = document.createElement("treeitem");
      let row  = document.createElement("treerow");
      let cell = document.createElement("treecell");

      cell.setAttribute("label", pc.host);
      row.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label",  pc.name);
      row.appendChild(cell);

      cell = document.createElement("treecell");
      cell.setAttribute("label",  pc.path);
      row.appendChild(cell);

      item.appendChild(row);
      item.setAttribute("accessibleType", "101F")
      this._treeChildren.appendChild(item);

      // To easily remove this item when a protected cookie is deleted
      pc.dom = item;
    }
  },

  remove: function() {
    var seln = this._treeView.view.selection;
    if (seln.count == 1) {
      var index = seln.currentIndex;

      var pCookie = this._pCookies[index];

      this._pCookies.splice(index, 1);

      // Remove protected cookie:
      CookieKeeper.removeProtectedCookie(pCookie);

      // Update ui:
      this._treeChildren.removeChild(pCookie.dom);
      var rowCount = this._pCookies.length;
      if (rowCount == 0) {
        this._buttonRemove.disabled = true;
        this._treeView.disabled     = true;
      } else {
        var newIndex = (index < rowCount ? index : rowCount - 1);
        seln.select(newIndex);
      }
    }
    this._treeView.focus();
  },

  // Sort the _pCookies list
  _sort: function(a, b) {
    var c = 0;

    switch (gCookieKeeperProtCookiesPrefs._sortColumn) {
      case 0:
        c = a.host.toLowerCase().localeCompare(b.host.toLowerCase());
        if (c != 0) break;
      case 1:
        c = a.name.toLowerCase().localeCompare(b.name.toLowerCase());
        if (c == 0) {
          c = a.host.toLowerCase().localeCompare(b.host.toLowerCase());
        }
        break;
      case 2:
        c = a.path.toLowerCase().localeCompare(b.path.toLowerCase());
        break;
    }

    if (gCookieKeeperProtCookiesPrefs._sortOrderAsc) {
      return c;
    } else {
      return -c;
    }
  },

  onSort: function(aEvent, aColumn) {
    if ((aEvent != null && aEvent.button != 0) || aColumn < 0 || aColumn > 2) {
      // param check (not usefull, but to be sure)
      return;
    }

    var colHeader = this._treeCols[aColumn];
    var column    = null;

    if (this._sortColumn == aColumn) {
      this._sortOrderAsc = !this._sortOrderAsc;
    } else {
      var oldColHeader = this._treeCols[this._sortColumn];

      this._sortOrderAsc = true;
      this._sortColumn   = aColumn;

      oldColHeader.removeAttribute("sortDirection");
    }
    colHeader.setAttribute("sortDirection", this._sortOrderAsc ? "ascending" : "descending");

    this._pCookies.sort(this._sort);

    // Not pretty, but easy to do
    while (this._treeChildren.firstChild) {
      this._treeChildren.removeChild(this._treeChildren.firstChild);
    }

    this._fillList();
  }

};
