/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2014 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Test Mozilla preferences.
 *
 * ************************************************************************
 */

var gTestMozPrefs = {

  _pService       : Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService),
  _pDefaultBranch : null,
  _pCurrentBranch : null,

  _appID        : "null",

  doTest: function() {
    var appinfos = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo).QueryInterface(Ci.nsIXULRuntime);
    this._appID = appinfos.ID;

    var showWarn = true;

    this._pDefaultBranch = this._pService.getDefaultBranch("");
    this._pCurrentBranch = this._pService.getBranch("");

    // Get default prefs:
    if (this._pDefaultBranch.prefHasUserValue(PREF_WARN_MOZ)) {
      showWarn = this._pDefaultBranch.getBoolPref(PREF_WARN_MOZ);
    }
    // Get preference:
    if (this._pCurrentBranch.prefHasUserValue(PREF_WARN_MOZ)) {
      showWarn = this._pCurrentBranch.getBoolPref(PREF_WARN_MOZ);
    }

    if (showWarn) {
      var appPrefInfos = APP_MOZ_CONST[this._appID];
      if (appPrefInfos.NAME == FIREFOX) {
        this._firefox();
      } else if (appPrefInfos.NAME == SEAMONKEY) {
        this._seamonkey();
      } else if (appPrefInfos.NAME == THUNDERBIRD) {
        this._thunderbird();
      } else {
        // Unknown Mozilla based application
        // Do nothing
      }
    }

  },

  onShowPrefs: function(aParam) {
    if (aParam == CK_PREFPANE_BROWSER) {
      let box = document.getElementById("boxFixPrefClear");
      box.hidden = true;
    }

    CookieKeeper.showPreferencesDialogue(window, aParam);
    gCookieKeeper._tree.focus();
  },

  // -------------------------------------
  // Mozilla preferences alert:
  _firefox: function() {
    const PRIVATEBROWSING           = APP_MOZ_CONST[this._appID].PRIVATEBROWSING;
    const SANITIZE_ON_SHUTDOWN      = APP_MOZ_CONST[this._appID].SANITIZE_ON_SHUTDOWN;
    const COOKIES_CLEAR_ON_SHUTDOWN = APP_MOZ_CONST[this._appID].COOKIES_CLEAR_ON_SHUTDOWN;
    const COOKIES_BEHAVIOR          = APP_MOZ_CONST[this._appID].COOKIES_BEHAVIOR;
    const COOKIES_LIFETIME          = APP_MOZ_CONST[this._appID].COOKIES_LIFETIME;
    // const COOKIES_LIFETIME_DAYS     = APP_MOZ_CONST[this._appID].COOKIES_LIFETIME_DAYS;

    // Mozilla Firefox 24.0 default preferences:
    var privateBrowsing = false;
    var doSanitize      = false;
    var clearOnShutdown = true;
    var acceptCookies   = true;
    var changeLifetime  = false;

    // default value:
    if (this._pDefaultBranch.prefHasUserValue(PRIVATEBROWSING)) {
      privateBrowsing = this._pDefaultBranch.getBoolPref(PRIVATEBROWSING);
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(PRIVATEBROWSING)) {
      privateBrowsing = this._pCurrentBranch.getBoolPref(PRIVATEBROWSING);
    }

    // default value:
    if (this._pDefaultBranch.prefHasUserValue(SANITIZE_ON_SHUTDOWN)) {
      doSanitize = this._pDefaultBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(SANITIZE_ON_SHUTDOWN)) {
      doSanitize = this._pCurrentBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    }
    if (doSanitize) {
      // default value:
      if (this._pDefaultBranch.prefHasUserValue(COOKIES_CLEAR_ON_SHUTDOWN)) {
        clearOnShutdown = this._pDefaultBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      }
      // preference:
      if (this._pCurrentBranch.prefHasUserValue(COOKIES_CLEAR_ON_SHUTDOWN)) {
        clearOnShutdown = this._pCurrentBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      }
    }

    // default value:
    var cookieBehavior = 0;
    if (this._pDefaultBranch.prefHasUserValue(COOKIES_BEHAVIOR)) {
      cookieBehavior = this._pDefaultBranch.getIntPref(COOKIES_BEHAVIOR);
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(COOKIES_BEHAVIOR)) {
      cookieBehavior = this._pCurrentBranch.getIntPref(COOKIES_BEHAVIOR);
    }
    acceptCookies = (cookieBehavior != MOZ_BEHAVIOR_DISABLED);

    // default value:
    if (this._pDefaultBranch.prefHasUserValue(COOKIES_LIFETIME)) {
      var mozCookieLifetime = this._pDefaultBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(COOKIES_LIFETIME)) {
      var mozCookieLifetime = this._pCurrentBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }

    if ((doSanitize && clearOnShutdown) || changeLifetime || privateBrowsing || !acceptCookies) {
      var box = document.getElementById("boxFixPrefClear");
      box.hidden = false;
    }

  },

  _seamonkey: function() {
    const SANITIZE_ON_SHUTDOWN      = APP_MOZ_CONST[this._appID].SANITIZE_ON_SHUTDOWN;
    const COOKIES_CLEAR_ON_SHUTDOWN = APP_MOZ_CONST[this._appID].COOKIES_CLEAR_ON_SHUTDOWN;
    const COOKIES_BEHAVIOR          = APP_MOZ_CONST[this._appID].COOKIES_BEHAVIOR;
    const COOKIES_LIFETIME          = APP_MOZ_CONST[this._appID].COOKIES_LIFETIME;

    // Mozilla SeaMonkey 2.21 default preferences:
    var doSanitize      = false;
    var clearOnShutdown = false;
    var acceptCookies   = true;
    var changeLifetime  = false;

    // default value:
    if (this._pDefaultBranch.prefHasUserValue(SANITIZE_ON_SHUTDOWN)) {
      doSanitize = this._pDefaultBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(SANITIZE_ON_SHUTDOWN)) {
      doSanitize = this._pCurrentBranch.getBoolPref(SANITIZE_ON_SHUTDOWN);
    }
    if (doSanitize) {
      // default value:
      if (this._pDefaultBranch.prefHasUserValue(COOKIES_CLEAR_ON_SHUTDOWN)) {
        clearOnShutdown = this._pDefaultBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      }

      // preference:
      if (this._pCurrentBranch.prefHasUserValue(COOKIES_CLEAR_ON_SHUTDOWN)) {
        clearOnShutdown = this._pCurrentBranch.getBoolPref(COOKIES_CLEAR_ON_SHUTDOWN);
      }
    }

    // default value:
    var cookieBehavior = 0;
    if (this._pDefaultBranch.prefHasUserValue(COOKIES_BEHAVIOR)) {
      cookieBehavior = this._pDefaultBranch.getIntPref(COOKIES_BEHAVIOR);
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(COOKIES_BEHAVIOR)) {
      cookieBehavior = this._pCurrentBranch.getIntPref(COOKIES_BEHAVIOR);
    }
    acceptCookies = (cookieBehavior != MOZ_BEHAVIOR_DISABLED);

    // default value:
    if (this._pDefaultBranch.prefHasUserValue(COOKIES_LIFETIME)) {
      var mozCookieLifetime = this._pDefaultBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(COOKIES_LIFETIME)) {
      var mozCookieLifetime = this._pCurrentBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }

    if ((doSanitize && clearOnShutdown) || changeLifetime || !acceptCookies) {
      var box = document.getElementById("boxFixPrefClear");
      box.hidden = false;
    }
  },

  _thunderbird: function() {
    const COOKIES_BEHAVIOR = APP_MOZ_CONST[this._appID].COOKIES_BEHAVIOR;
    const COOKIES_LIFETIME = APP_MOZ_CONST[this._appID].COOKIES_LIFETIME;

    // Mozilla Thunderbird 24.0 default preferences:
    var acceptCookies   = true;
    var changeLifetime  = false;

    // default value:
    var cookieBehavior = 0;
    if (this._pDefaultBranch.prefHasUserValue(COOKIES_BEHAVIOR)) {
      cookieBehavior = this._pDefaultBranch.getIntPref(COOKIES_BEHAVIOR);
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(COOKIES_BEHAVIOR)) {
      cookieBehavior = this._pCurrentBranch.getIntPref(COOKIES_BEHAVIOR);
    }
    acceptCookies = (cookieBehavior != MOZ_BEHAVIOR_DISABLED);

    // default value:
    if (this._pDefaultBranch.prefHasUserValue(COOKIES_LIFETIME)) {
      var mozCookieLifetime = this._pDefaultBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }
    // preference:
    if (this._pCurrentBranch.prefHasUserValue(COOKIES_LIFETIME)) {
      var mozCookieLifetime = this._pCurrentBranch.getIntPref(COOKIES_LIFETIME);
      changeLifetime = mozCookieLifetime == MOZ_ACCEPT_SESSION;
    }

    if (changeLifetime || !acceptCookies) {
      var box = document.getElementById("boxFixPrefClear");
      box.hidden = false;
    }
  }
}