/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2014 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Locale (l10n) management from string bundle.
 *
 * ************************************************************************
 */

"use strict";

const DEFAULT_LOCALE = "en";

var l10n = {
  initDone : false,
  addonURI : null,
  filename : null,
  locale   : null,
  bundle   : null,
  dBundle  : null, // default bundle (english)

  _getBundle: function(_locale) {
    let e = null;
    let p = this.addonURI + "chrome/locale/" + _locale + "/" + this.filename;
    let b = Services.strings.createBundle(p);
    try {
      e = b.getSimpleEnumeration();
      return b;
    } catch (ex) {
      // log.w("Bundle for '" + _locale + "' not found");
    }
    return null;
  },

  init: function(_addonURI, _filename) {
    if (this.initDone == false) {
      this.addonURI = _addonURI;
      this.filename = _filename;
      this.dBundle  = this._getBundle(DEFAULT_LOCALE);
      let chrome    = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULChromeRegistry);
      let locale    = chrome.getSelectedLocale("global").toLowerCase();
      let bundle    = this._getBundle(locale);

      if (bundle == null) {
        // FIXME
        locale = locale.split("-")[0];
        bundle = this._getBundle(locale);
      }
      if (bundle == null) {
        locale = DEFAULT_LOCALE;
        bundle = this.dBundle;
      }
      this.locale   = locale;
      this.bundle   = bundle;
      this.initDone = true;

      // log.i("l10n: init done (using locale '" + this.locale + "')");
    }
  },

  uninit: function() {
    this.addonURI = null;
    this.filename = null;
    this.dBundle  = null;
    this.bundle   = null;
    this.locale   = null;
    this.initDone = false;
    // log.i("l10n: uninit done");
  }
};

function tr(textId) {
  let text = null;

  if (typeof(textId) == "string") {
    try {
      text = l10n.bundle.GetStringFromName(textId);
    } catch (ex) {
      Cu.reportError(ex);
    }

    if (text == null) {
      try {
        text = l10n.dBundle.GetStringFromName(textId);
      } catch (ex) {
        Cu.reportError(ex);
      }
    }
  }

  if (text == null) {
    text = "" + textId;
  }

  return text;
}
