/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2014 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Main lib component
 *
 * ************************************************************************
 */

const RESOURCE = "cookiekeeper";

let ckMain = {

  init: function() {
    // Aliasing the Public.jsm API
    // https://developer.mozilla.org/en-US/docs/Mozilla/JavaScript_code_modules/Using
    let resource = Services.io.getProtocolHandler("resource").QueryInterface(Ci.nsIResProtocolHandler);
    let uriInstP = Services.io.newFileURI(addonData.installPath);

    if (addonData.installPath.isDirectory()) {
      var alias = Services.io.newURI(uriInstP.spec + "chrome/modules/", null, null);
    } else {
      var alias = Services.io.newURI("jar:" + uriInstP.spec + "!/chrome/modules/", null, null);
    }

    resource.setSubstitution(RESOURCE, alias);
    // Init the CookieKeeper resource datas:
    Cu.import("resource://cookiekeeper/cookiekeeper.jsm");
    CookieKeeper.init();

    // Init the locale bundle
    l10n.init(addonData.resourceURI.spec, "cookiekeeper.properties");

    // Set the toolbar icon + tools menu
    ckUi.init();

    // TODO: When bug #564675 is implemented this will no longer be needed
    // Always set the default prefs, because they disappear on restart
    // See: https://bugzilla.mozilla.org/show_bug.cgi?id=564675
    ckPreferences.setDefaultPrefs(addonData);
  },

  uninit: function() {
    ckUi.uninit();

    // Clear unwanted cookies at shutdown
    if (APP_SHUTDOWN == currReason) {
      ckStorageCleaner.clearAllCookies(true);
    } // APP_SHUTDOWN

    // Unload the module
    CookieKeeper.uninit();
    Cu.unload("resource://cookiekeeper/cookiekeeper.jsm");
    // Remove the resources alias
    var resProt = Services.io.getProtocolHandler("resource")
                          .QueryInterface(Components.interfaces.nsIResProtocolHandler);
    resProt.setSubstitution(RESOURCE, null);
  }
}
