/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2014 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Clean cookies at exit.
 *
 * ************************************************************************
 */

"use strict";

var ckStorageCleaner = {

  _sCookieManager : Cc["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager2),

  /// @brief Load cookies datas
  /// @param clearCookies (boolean) if true, the return array will contain only the protected cookies.
  /// @return an array of cookies.
  _loadCookies: function(clearCookies) {
    var e = this._sCookieManager.enumerator;
    var cookieList = [];
    var protList   = [];

    if (clearCookies) {
      protList = CookieKeeper.loadProtectedCookies();
    }

    while (e.hasMoreElements()) {
      var cookie = e.getNext();
      if (cookie && cookie instanceof Ci.nsICookie2) {
        if (clearCookies) {
          if (CookieKeeper.isProtectedCookie(cookie, protList)) {
            cookieList.push(cookie);
          }
        } else {
          cookieList.push(cookie);
        }
      }
    }

    return cookieList;
  },

  /// @brief Clear only unwanted cookies
  _clearUnwantedCookies: function() {
    var protList = CookieKeeper.loadProtectedCookies();
    var clearList = [];
    var e = this._sCookieManager.enumerator;

    log.d("Loaded " + protList.length + " protected cookies");

    while (e.hasMoreElements()) {
      var cookie = e.getNext();
      if (cookie && cookie instanceof Ci.nsICookie2) {
        if (!CookieKeeper.isProtectedCookie(cookie, protList)) {
          clearList.push(cookie);
        }
      }
    }

    for (var i=0; i<clearList.length; i++) {
      var cookie = clearList[i];
      this._sCookieManager.remove(cookie.host, cookie.name, cookie.path, false);
    }

  },

  /// Main function
  clearAllCookies: function(atExit) {
    var psvc = Services.prefs;
    var clearCookies = true;
    var clearDOM     = true;

    if (atExit) {
      clearCookies = psvc.prefHasUserValue(PREF_CLEAR_COOKIES_AT_EXIT) ? psvc.getBoolPref(PREF_CLEAR_COOKIES_AT_EXIT) : false;
      clearDOM     = psvc.prefHasUserValue(PREF_CLEAR_DOM_AT_EXIT)     ? psvc.getBoolPref(PREF_CLEAR_DOM_AT_EXIT)     : false;
    }

    var cookieList   = [];

    log.d("clearAllCookies()");

    if (clearDOM) {
      // Get all cookies, perform a removeAll() which currently clear the DOM storage too, and reinsert the cookies
      cookieList = this._loadCookies(clearCookies);
      this._sCookieManager.removeAll();
    } else if (clearCookies) {
      // Clear only the cookies we do not want to keep
      this._clearUnwantedCookies();
    } else {
      log.d("Nothing is cleared");
    }

    // Reinsert cookies we wants to keep
    for (var i=0; i<cookieList.length; i++) {
      var cookie = cookieList[i];
      this._sCookieManager.add(cookie.host, cookie.path, cookie.name, cookie.value, cookie.isSecure, cookie.isHttpOnly, cookie.isSession, cookie.expires);
    }
  }

} // gStorageCleaner
